
/*  @(#)header.c 1.4 95/02/06
 *
 *  Various routines associated with the header popup.
 *
 *  Copyright (c) Rich Burridge.    All rights reserved.
 *
 *  Permission is given to distribute these sources, as long as the
 *  copyright messages are not removed, and no monies are exchanged.
 *
 *  No responsibility is taken for any errors or inaccuracies inherent
 *  either to the comments or the code of this program, but if
 *  reported to me then an attempt will be made to fix them.
 */

#include "mptool.h"
#include <Xm/List.h>

char *hdrs[MAXSTDHDRS] ;   /* The array of standard mail/news headers. */
Widget hpopup ;            /* Mail/News header popup.                   */
Widget *hwarr ;                                                        

static char *get_list_str  (int) ;                                      
static void hdr_select     (Widget, XtPointer, XmListCallbackStruct *) ;
static void hdr_text       (Widget, XtPointer, XmTextVerifyCallbackStruct *) ;


void
do_hdrs_add(void)
{
  char buf[MAXLINE] ;
  int n ;

  STRCPY(buf, get_str_value(F_HDRS, WI_HDR_LIST_ADD)) ;
  n = strlen(buf) ;
  if (buf == NULL || !n)
    notice(hwarr[WI_HEADER_FRAME], mptool_strs[(int) S_NOTHING]) ;
  else
    { 
      if (buf[n-1] != ':') STRCAT(buf, ":") ;
      add_list_item(hwarr[WI_HDR_LIST], 0, buf) ;
      set_str_value(F_HDRS, WI_HDR_LIST_ADD, "") ;
      set_list_position(hwarr[WI_HDR_LIST], 0) ;
    }
}


void
do_hdrs_apply(void)
{
  char *liststr ;
  int found, i, j, n ;
  int nrows = get_scroll_nrows(hwarr[WI_HDR_LIST]) ;

  set_int_opt_val(O_ALLHDRS, get_bool_value(F_HDRS, WI_HDRS_ALL)) ;

  for (i = 0; i < int_opt_val(O_REMHDRN); i++)
    free_hdr(H_REMOVE, i) ;
  set_int_opt_val(O_REMHDRN, 0) ;
  for (i = 0; i < MAXSTDHDRS; i++)
    {
      found = FALSE ;
      for (j = 0; j < nrows && found == FALSE; j++)
        if (EQUAL(hdrs[i], get_list_str(j))) found = TRUE ;
      if (found == FALSE)
        {
          get_hdr_arg(hdrs[i], H_REMOVE, &n) ;
          set_int_opt_val(O_REMHDRN, n) ;
        }
    }
 
  for (i = 0; i < int_opt_val(O_ADDHDRN); i++) free_hdr(H_ADD, i) ;
  set_int_opt_val(O_ADDHDRN, 0) ;
  for (i = 0; i < nrows; i++)
    {
      found = FALSE ;
      liststr = get_list_str(i) ;
      for (j = 0; j < MAXSTDHDRS && found == FALSE; j++)
        if (EQUAL(liststr, hdrs[j])) found = TRUE ;
      if (found == FALSE)
        {
          get_hdr_arg(liststr, H_ADD, &n) ;
          set_int_opt_val(O_ADDHDRN, n) ;
        }
    }
}


void
do_hdrs_dismiss(void)
{
  XtUnmapWidget(hpopup) ;
  XtUnmanageChild(hpopup) ;
}


void
do_hdrs_remove(void)
{
  int i, n, *pos_list ;

  if (XmListGetSelectedPos(hwarr[WI_HDR_LIST], &pos_list, &n))
    {
      for (i = 0; i < n; i++)
        XmListDeleteItemsPos(hwarr[WI_HDR_LIST], 1, pos_list[i]) ;
      set_active(F_HDRS, WI_HDR_REMOVE_BUTTON, False) ;
    }
  else notice(hwarr[WI_HEADER_FRAME], mptool_strs[(int) S_NONE]) ;
}


void
do_hdrs_reset(void)
{
  int i, pos ;

  set_bool_value(F_HDRS, WI_HDRS_ALL, int_opt_val(O_ALLHDRS)) ;
  XmListDeleteAllItems(hwarr[WI_HDR_LIST]) ;
  for (i = 0; i < MAXSTDHDRS; i++)
    add_list_item(hwarr[WI_HDR_LIST], 0, hdrs[i]) ;

  for (i = 0; i < int_opt_val(O_ADDHDRN); i++)
    if (item_exists(hwarr[WI_HDR_LIST], get_hdr(H_ADD, i)) == 0)
      add_list_item(hwarr[WI_HDR_LIST], 0, get_hdr(H_ADD, i)) ;

  for (i = 0; i < int_opt_val(O_REMHDRN); i++)
    if ((pos = item_exists(hwarr[WI_HDR_LIST], get_hdr(H_REMOVE, i))))
      delete_list_item(hwarr[WI_HDR_LIST], pos) ;
  do_toggle_hdr_list() ;
}


void
do_show_headers(void)
{
  if (!hpopup)
    {
      hpopup = tu_header_frame_widget("headers", warr[WI_MAIN_FRAME],
                                      &hwarr) ;
      add_delete_callback(hpopup) ;
      XtAddCallback(hwarr[WI_HDR_LIST], XmNbrowseSelectionCallback,
                (XtCallbackProc) hdr_select, NULL) ;
      XtAddCallback(hwarr[WI_HDR_LIST_ADD], XmNmodifyVerifyCallback,
                                (XtCallbackProc) hdr_text, 0) ;

      add_help(hwarr[WI_HEADER_FORM],       wi_header_frame_help) ;
      add_help(hwarr[WI_HDRS_ALL],          wi_headers_all_help) ;
      add_help(hwarr[WI_HDR_LIST],          wi_hdr_list_help) ;
      add_help(hwarr[WI_HDR_LIST_ADD],      wi_hdr_add_help) ;
      add_help(hwarr[WI_HDR_ADD_LABEL],     wi_hdr_add_help) ;
      add_help(hwarr[WI_HDR_APPLY],         wi_hdr_apply_help) ;
      add_help(hwarr[WI_HDR_DISMISS],       wi_hdr_dismiss_help) ;
      add_help(hwarr[WI_HDR_ADD_BUTTON],    wi_hdr_add_help) ;
      add_help(hwarr[WI_HDR_REMOVE_BUTTON], wi_hdr_remove_help) ;
      add_help(hwarr[WI_HDR_RESET],         wi_hdr_reset_help) ;
      set_active(F_HDRS, WI_HDR_ADD_BUTTON,    False) ;
      set_active(F_HDRS, WI_HDR_REMOVE_BUTTON, False) ;
      do_hdrs_reset() ;                               /* Headers to print. */
    }
  show_popup(hpopup) ;
}


void
do_toggle_hdr_list(void)
{
  int n, *pos_list ;
  char *str    = get_str_value(F_HDRS, WI_HDR_LIST_ADD) ;
  int value    = get_bool_value(F_HDRS, WI_HDRS_ALL) ;
  int selected = XmListGetSelectedPos(hwarr[WI_HDR_LIST], &pos_list, &n) ;

  set_active(F_HDRS, WI_HDR_LIST,          !value) ;
  set_active(F_HDRS, WI_HDR_LIST_ADD,      !value) ;
  set_active(F_HDRS, WI_HDR_ADD_LABEL,     !value) ;
  set_active(F_HDRS, WI_HDR_ADD_BUTTON,    False) ;
  set_active(F_HDRS, WI_HDR_REMOVE_BUTTON, False) ;

  if (!value && str && *str) set_active(F_HDRS, WI_HDR_ADD_BUTTON,    True) ;
  if (!value && selected)    set_active(F_HDRS, WI_HDR_REMOVE_BUTTON, True) ;
}


static char *
get_list_str(int n)
{
  char *label ;
  int nrows = get_scroll_nrows(hwarr[WI_HDR_LIST]) ;
  XmString *strlist ;

  XtVaGetValues(hwarr[WI_HDR_LIST], XmNitems, &strlist, 0) ;
  XmStringGetLtoR(strlist[n], XmSTRING_DEFAULT_CHARSET, &label) ;
  return(label) ;
}


void
Header(void)
{
  int i ;

  hwarr  = NULL ;                /* No widgets defined initially. */
  hpopup = NULL ;                /* No header popup window initially. */
  for (i = 0; i < MAXSTDHDRS; i++) hdrs[i] = NULL ;
}


static void
hdr_select(Widget w, XtPointer p, XmListCallbackStruct *s)
{
  set_active(F_HDRS, WI_HDR_REMOVE_BUTTON, True) ;
}


static void
hdr_text(Widget w, XtPointer p, XmTextVerifyCallbackStruct *s)
{
  set_active(F_HDRS, WI_HDR_ADD_BUTTON, True) ;
}


void
init_headers(void)
{
  int i = 0 ;

  read_str(&hdrs[i++], APP_FROMHDR) ;
  read_str(&hdrs[i++], APP_TOHDR) ;
  read_str(&hdrs[i++], CCHDR) ;
  read_str(&hdrs[i++], CONTENT_LEN) ;
  read_str(&hdrs[i++], DATEHDR) ;
  read_str(&hdrs[i++], FROMHDR) ;
  read_str(&hdrs[i++], FROM_HDR) ;
  read_str(&hdrs[i++], NEWSGROUPHDR) ;
  read_str(&hdrs[i++], NEWSGROUPSHDR) ;
  read_str(&hdrs[i++], REPLYHDR) ;
  read_str(&hdrs[i++], SUBJECTHDR) ;
  read_str(&hdrs[i++], TOHDR) ;
}
